import json
import jwt
import requests
from typing import Dict, Optional
from rich.console import Console
from aichor_cli.config import get_config_paths

console = Console()
config_path, config_dir = get_config_paths()
class Client:
    def __init__(self, config_path: str):
        self.config_path = config_path
        self.token = self.get_access_token()
        self.base_url = self.get_base_url()
        self.headers = {'Authorization': f'Bearer {self.token}'}

    def get_access_token(self) -> str:
        with open(self.config_path, "r") as config_file:
            config = json.load(config_file)
            token = config["credentials"]["token"]
            return token
        
    def decode_jwt(self) -> dict:
        try:
            payload = jwt.decode(self.token, options={"verify_signature": False})
            return payload
        except jwt.PyJWTError as e:
            raise Exception(f"Failed to decode JWT: {e}")  

    def get_user_roles(self) -> list:
        payload = self.decode_jwt()
        roles_claim = "http://schemas.microsoft.com/ws/2008/06/identity/claims/role"     
        roles = payload.get(roles_claim, [])       
        if not roles:
            raise Exception("No roles found in JWT token.")      
        return roles
        
    def get_base_url(self) -> str:
        with open(self.config_path, "r") as config_file:
            config = json.load(config_file)
            qarth_service = config.get('qarthService')
            base_url = config.get('baseURL')
            stage = config.get('stage')
            base_url = f"{qarth_service}{base_url}/{stage}api/v1"
        return base_url

    def handle_request(self, endpoint: str, method: str, data: Optional[Dict] = None) -> Dict:
        try:
            url = f'{self.base_url}{endpoint}'
            response = requests.request(method=method, url=url, headers=self.headers, json=data)
            response.raise_for_status()
            response_json = response.json()
            if response_json.get("status") != "success":
                raise Exception("Error: API request failed")
            return response_json
        except requests.exceptions.RequestException as e:
            console.print("[red][bold]Error accessing API:[/bold][/red]", e)
            return {}
