import os
import tempfile
import requests
import datetime
from dateutil import parser
from typing import Dict, List, Optional
from rich.table import Table
from rich.console import Console
from rich import box
from aichor_cli.config import get_config_paths
from aichor_cli.utils import zip_directory
from aichor_cli.client.client import Client
from aichor_cli.client.project import Project

config_path, config_dir = get_config_paths()
console = Console()
class Experiment:
    def __init__(self,client: Client,project: Project):
        self.client = client
        self.project = project

    def get_experiment_id(self, project_id: str, page_number: int = 1, experiments_per_page: int = 10) -> Optional[str]:
        if project_id:
            data = self.client.handle_request(
            f'/projects/{project_id}/experiments?pageNumber={page_number}&experimentsPerPage={experiments_per_page}',
            'get')
            if data.get('result') and data['result'].get('items'):
                return data['result']['items'][0]['experimentId']
            else:
                print("No experiments found")
                return None
        else:
            print("Project ID is required")
            return None
            
    def get_experiments_by_project_name(self, project_name: str, page_number: int = 1, experiments_per_page: int = 10) -> List[Dict]:
        extracted_data = []
        try:
            project_id = self.project.get_project_id(project_name)
            if project_id == "":
                console.print("[red][bold]The project does not exist[/bold][/red]")
                return extracted_data
            experiments_data = self.client.handle_request(
                f'/projects/{project_id}/experiments?pageNumber={page_number}&experimentsPerPage={experiments_per_page}',
                'get')
            if experiments_data["status"] != "success":
                raise Exception("Error: API request failed")
            items = experiments_data['result']['items']
            if not items:
                return extracted_data
            for item in items:
                extracted_item = {
                    "experiment_id": item["experimentId"],
                    "branch_name": item["namespace"],
                    "commit_message": item["commitMessage"],
                    "step_status": item["stepStatus"],
                    "experiment_status": item["experimentStatus"],
                    "step": item["step"],
                    "created_on": item["createdOn"],
                }
                extracted_data.append(extracted_item)
        except requests.exceptions.RequestException as e:
            print("Error making API request:", e)
        return extracted_data
    
    def get_experiments_by_project_id(self, project_id: str, page_number: int = 1, experiments_per_page: int = 10) -> List[Dict]:
        if not project_id:
            print("Project ID is required")
            return []
        data = self.client.handle_request(f'/projects/{project_id}/experiments?pageNumber={page_number}&experimentsPerPage={experiments_per_page}', 'get')
        if 'result' in data and 'items' in data['result']:
            items = data['result']['items']
            if not items:
                return []
            extracted_data = []
            for item in items:
                extracted_data.append(
                   {
                        "experiment_id": item["experimentId"],
                        "branch_name": item["namespace"],
                        "commit_message": item["commitMessage"],
                        "step_status": item["stepStatus"],
                        "experiment_status": item["experimentStatus"],
                        "step": item["step"],
                        "created_on": item["createdOn"],
                    }
                 )
            return extracted_data
        return []
          
    def get_experiment_by_id(self, project_id: str, experiment_id: str) -> Dict:
        data = self.client.handle_request(f'/projects/{project_id}/experiments/{experiment_id}', 'get')
        if data:
            return data
        else:
            return None        
        
    def get_experiment_status(self, project_id: str, experiment_id: str) -> str:
        experiment_data = self.get_experiment_by_id(project_id, experiment_id)
        if experiment_data and experiment_data.get('result'):
            return experiment_data['result'].get('experimentStatus')
        return None

    def get_experiment_date(self, project_id: str, experiment_id: str, page_number: int = 1, experiments_per_page: int = 10) -> Optional[str]:
        experiments = self.get_experiments_by_project_id(project_id, page_number, experiments_per_page)
        for item in experiments:
            if item.get('experiment_id') == experiment_id:
                return item.get('created_on')
        console.print(f"No experiment found with ID '{experiment_id}'.")
        return None
    
    def is_experiment_old(self, created_on: str) -> bool:
        if created_on:
            creation_date = parser.isoparse(created_on).astimezone(datetime.timezone.utc)
            current_date = datetime.datetime.utcnow().replace(tzinfo=datetime.timezone.utc)
            time_difference = current_date - creation_date
            return time_difference.days > 7
        return False

    def cancel_experiment(self,project_id: str, experiment_id: str) -> int:
        data = self.client.handle_request(f'/projects/{project_id}/experiments/{experiment_id}/cancel', 'put')
        if data:
            return 0
        return -1 
        
    def resubmit(self,project_id,experiment_id: str):
        endpoint = f'/projects/{project_id}/experiments/{experiment_id}/resubmit'
        data = self.client.handle_request(endpoint, 'patch')
        if data:
            console.print("[green][bold]Experiment resubmitted successfully.[/bold][/green]")
        else:
            console.print("[red]][bold]Resubmit failed[/bold][/red]")    

    def resubmit_experiments(self, project_id: str, experiment_id: Optional[str] = None):
        if project_id is None:
            project_id = self.project.load_default_project(config_path)
            if project_id is None:
                return       
        if experiment_id:
            self._resubmit_specific_experiment(project_id, experiment_id)
        else:
            self._select_and_resubmit_experiments(project_id)

    def _resubmit_specific_experiment(self, project_id: str, experiment_id: str):
        experiment_data = self.get_experiment_by_id(project_id, experiment_id)
        if experiment_data is None:
            console.print("[red][bold]Error accessing API: please verify your experiment ID or if you have not been logged in[/bold][/red]")
            return
        if experiment_data.get('result') is None:
            console.print("[red][bold]Experiment not found for the given project and experiment ID[/bold][/red]")
            return
        experiment_status = self.get_experiment_status(project_id, experiment_id)
        experiment_date = self.get_experiment_date(project_id, experiment_id)
        if experiment_status == "Processing":
            console.print("[red][bold]Cannot resubmit an experiment that is running. Resubmission aborted.[/bold][/red]")
            return
        if  self.is_experiment_old(experiment_date):
            console.print("[red][bold]Experiment cannot be resubmitted as it was created over 7 days ago. Resubmission aborted.[/bold][/red]")
            return
        self.resubmit(project_id, experiment_id)

    def _select_and_resubmit_experiments(self, project_id: str):
        indexed_experiments, total_experiments_count, project_contains_experiments = self._list_experiments(project_id)
        if project_contains_experiments:
            self._resubmit_selected_experiment(project_id, indexed_experiments, total_experiments_count)

    def _list_experiments(self, project_id: str):
        indexed_experiments = {}
        total_experiments_count = 0
        project_contains_experiments: bool = False
        page_number = 1
        while True:
            experiments: list[dict] = self.get_experiments_by_project_id(project_id, page_number=page_number, experiments_per_page=10)
            if not experiments:
                break
            project_contains_experiments = True
            table = Table(title=f"[bold magenta]Experiments - Page {page_number}[/bold magenta]", style="bright_yellow", border_style="dim",box=box.HORIZONTALS,pad_edge=False,collapse_padding=True )
            table.add_column("Index")
            table.add_column("Experiment ID")
            table.add_column("Commit Message")
            table.add_column("Status")
            table.add_column("Date")
            for index, experiment in enumerate(experiments, start=total_experiments_count + 1):
                indexed_experiments[index] = experiment
                commit_message_shortened = experiment['commit_message'].strip()[:50] + ('...' if len(experiment['commit_message']) > 50 else '')
                table.add_row(str(index), experiment['experiment_id'].strip(), commit_message_shortened, experiment['experiment_status'].strip(), experiment['created_on'][:10].strip())
                total_experiments_count += 1
            console.print(table)
            next_action = input("Press 'n' to skip viewing more experiments, or Enter to view more experiments: ")
            if next_action.lower() == 'n':
                console.print("[blue]Skipping viewing more experiments...[/blue]")
                break
            elif next_action.strip() == '':
                page_number += 1
            else:
                console.print("[red][bold]Invalid input[/bold][/red]")
                break
        return indexed_experiments, total_experiments_count, project_contains_experiments

    def _resubmit_selected_experiment(self, project_id: str, indexed_experiments: dict, total_experiments_count: int):
        while True:
            try:
                choice = input("Enter the number of the experiment to resubmit (or q to cancel): ")
                if choice.lower() == "q":
                    console.print("[red][bold]Resubmission canceled.[/bold][/red]")
                    break
                if int(choice) > total_experiments_count:
                    console.print("[red][bold]Invalid choice. Please enter a valid number.[/bold][/red]")
                    break
                else:
                    chosen_experiment = indexed_experiments[int(choice)]
                    experiment_id = chosen_experiment['experiment_id']
                    chosen_experiment_page = 1 + (int(choice) - 1) // 10
                    experiment_status = self.get_experiment_status(project_id, experiment_id)
                    experiment_date = self.get_experiment_date(project_id, experiment_id, page_number=chosen_experiment_page)
                    if experiment_status.upper() in ("CANCELLED", "FAILED","SUCCEEDED"):
                        if  self.is_experiment_old(experiment_date):
                            console.print("[red][bold]Experiment cannot be resubmitted as it was created over 7 days ago. Resubmission aborted.[/bold][/red]")
                        else:
                            try:
                                self.resubmit(project_id, experiment_id)
                            except Exception as e:                              
                                console.print(f"[red][bold]Failed to resubmit experiment: {e}[/bold][/red]")
                    else:
                        console.print("[red][bold]Cannot resubmit an experiment that is running. Resubmission aborted.[/bold][/red]")
                    break
            except ValueError:
                console.print("[red][bold]Invalid choice. Please enter a valid number.[/bold][/red]")

    def get_signed_url(self,project_name,headers):
        data = {
            "projectName": project_name,
        }
        response = self.client.handle_request( '/cli/upload-url', 'post',data)
        if response:
                return response["result"]["signedUrl"], response["result"]["triggerId"]
        else:
            console.print("failed to get signed url")
            return None ,None

    def upload_file(self,zip_path, upload_url):
        try:
            with open(zip_path, 'rb') as f:
                put_response = requests.put(upload_url, data=f)
                put_response.raise_for_status()  # Raise HTTPError for bad responses
                if put_response.status_code == 200:
                    console.print("[green][bold]File uploaded successfully.[/bold][/green]")
                    return True
                else:
                    console.print(f"[red][bold]Failed to upload file. Status code: {put_response.status_code}[red][bold]")
                    return False
        except (requests.RequestException, requests.HTTPError, ConnectionError) as e:
            print(f"Error while uploading file: {e}")
            return False

    def trigger_experiment(self,project_name, trigger_id,message, headers):
        trigger_data = {
            "projectName": project_name,
            "triggerId": trigger_id,
            "message": message
        }
        response = self.client.handle_request('/cli/trigger', 'post', trigger_data)
        if response:
            console.print("[green][bold]Experiment triggered successfully.[/bold][/green]")
            return True
        console.print("[red][bold]Failed to trigger experiment.[red][bold]")
        return False

    def submit_experiment(self,project_name, directory_path, message):
        token = self.client.get_access_token()
        headers = {"Authorization": f"Bearer {token}"}
        # Resolve the directory path
        directory_path = os.path.abspath(directory_path)
        # Create a temporary directory to store the zip file
        with tempfile.TemporaryDirectory() as temp_dir:
            zip_path = os.path.join(temp_dir, f"{project_name}.zip")
            # ZIP the directory
            zip_directory(directory_path, zip_path)            
            # Request signed upload URL
            upload_url, trigger_id = self.get_signed_url(project_name,headers)
            if not upload_url or not trigger_id:
                return            
            # Upload the ZIP file
            if self.upload_file(zip_path, upload_url):
                # Trigger the experiment
                self.trigger_experiment(project_name, trigger_id,message, headers)

