import json
import click
from rich.table import Table
from rich.console import Console
from rich import box
from rich.text import Text
from aichor_cli.client.experiment import Experiment
from aichor_cli.client.project import Project
from aichor_cli.utils import set_default_project_in_config
from aichor_cli.config import get_config_paths
from aichor_cli.client.client import Client

console = Console()
config_path, config_dir = get_config_paths()
client_instance= Client(config_path)
project_instance =  Project(client_instance) 
experiment_instance = Experiment(client_instance,project_instance)
def display_projects(project_id):
    projects_info = project_instance.list_projects()
    if projects_info:
        if project_id:
            display_project(project_id)
            return
        table = Table(
            show_header=True,header_style="bold magenta",title="Projects",collapse_padding=True,box=box.HORIZONTALS,style="bright_yellow",title_style="italic")
        table.add_column("Index")
        table.add_column("Default")
        table.add_column("Name")
        table.add_column("Monthly Cost")
        table.add_column("Status")
        table.add_column("Date", style="dim", width=12)
        for col in table.columns:
            col.width = 20
        for project in projects_info:
            status_text = f"[green]{project['status']}[/green]" if project["status"].lower() == "ready" else project["status"]
            table.add_row(str(project["index"]),project["default"],project["name"], f"${project['totalCost']:.2f}", status_text, project["createdOn"][:10])
        console.print(table)
    else:
            console.print("[bold red]Error accessing API: Unable to retrieve project list.[/bold red]")
    

def display_experiments(project_id, experiment_id):
    if project_id is None:
        project_id = project_instance.load_default_project(config_path)
        if project_id is None:
            return
    if experiment_id:
        experiment_data = experiment_instance.get_experiment_by_id(project_id, experiment_id)
        if experiment_data is not None:
            if experiment_data['result'] is not None:
                display_experiment(experiment_data)
            else:
                console.print("[red][bold]Experiment not found for the given project and experiment ID[/bold][/red]")
        else:
            console.print("[red][bold]Experiment ID does not exist[/bold][/red]")
    else:
        page_number = 1
        while True:
            experiments_data = experiment_instance.get_experiments_by_project_id(project_id, page_number)
            if not experiments_data:
                console.print("[red][bold]No experiments found for the given project[/bold][/red]")
                return
            table = Table(
                show_header=True,
                header_style="cyan",
                title=f"Experiments - Page {page_number}",
                collapse_padding=True,
                box=box.HORIZONTALS,
                style="bright_green",
                title_style="italic",
                pad_edge=False,
            )
            table.add_column("Experiment Id")
            table.add_column("Commit Message", width=40)
            table.add_column("Step")
            table.add_column("Status", width=30)
            table.add_column("Date", style="dim", width=25)
            for experiment in experiments_data:
                table.add_row(
                    experiment['experiment_id'].strip(),
                    experiment['commit_message'].strip()[:30] + ('...' if len(experiment['commit_message']) > 40 else ''),
                    experiment['step'].strip(),
                    experiment['experiment_status'].strip(),
                    experiment['created_on'][:10].strip())
            console.print(table)
            if len(experiments_data) == 10:
                user_input = input("Press Enter to view more experiments or 'q' to quit: ")
                if user_input.lower() == 'q':
                    break
                elif user_input == '':
                    page_number += 1
                else:
                    console.print("[red][bold]Invalid input. Please Press Enter to view more experiments or 'q' to quit.[/bold][/red]")
            else:
                console.print("No more experiments to show.")
                break

def display_experiment(experiment_data: dict):
    if experiment_data:
        table = Table(
                show_header=True,
                header_style="cyan",
                title="Experiments Details",
                collapse_padding=True,
                box=box.HORIZONTALS,
                style="bright_green",
                title_style="italic"
            )  
        table.add_column("Experiment ID")
        table.add_column("Commit Message", width=40)
        table.add_column("Status")
        table.add_column("Date", style="dim", width=25)
        table.add_row(
            experiment_data["result"]["experimentId"],
            experiment_data["result"]["commitMessage"][:30] + ('...' if len(experiment_data["result"]["commitMessage"]) > 40 else ''),
            experiment_data["result"]["experimentStatus"],
            experiment_data["result"]["createdOn"][:10],
        )
        
        console.print(table)
    else:
        console.print("[red][bold]Experiment data is empty or invalid[/bold][/red]")            

def display_project(project_id):
    project= project_instance.get_project_info(project_id)
    if project:
        default_project_name = None
        with open(config_path, "r") as config_file:
            config = json.load(config_file)
            if 'context' in config and 'projectName' in config['context']:
                default_project_name = config['context']['projectName']
        table = Table(show_header=True, header_style="bold cyan")
        table.add_column("Default")
        table.add_column("Name")
        table.add_column("Monthly Cost")
        table.add_column("Status")
        table.add_column("Date")
        status_text = f"[green]{project['status']}[/green]" if project["status"].lower() == "ready" else project["status"]
        table.add_row(
                Text("⭐", style="yellow") if project["name"] == default_project_name else "",
                project["name"],
                str(project["billingCost"]["totalCost"]),
                status_text,
                project["createdOn"][:10]
            )
        console.print(table)
    else:
        console.print("Project not found or Invalid ID project", style="bold red")

def set_project(project_id):
    if project_id is not None:
        # Retrieve project name using project ID
        project_name = project_instance.get_project_name(project_id)
        if project_name:
            # Call function to set default project using project_id and project_name
            set_default_project_in_config(config_path,project_name, project_id)
            console.print(f"Default project set to: {project_name} ({project_id})", style="bold green")
        else:
            console.print("Project ID not found or error retrieving project name.", style="bold red")
    else:
        projects_info = project_instance.list_projects()
        if projects_info:
            # Display projects info
            display_projects(projects_info,project_id)
            # Prompt user to choose default project
            default_project_index = click.prompt("Choose default project", default=None)
            try:
                default_project_index = int(default_project_index) - 1
                if 0 <= default_project_index < len(projects_info):
                    default_project = projects_info[default_project_index]
                    set_default_project_in_config(config_path,default_project["name"], default_project["id"])
                    console.print(f"Default project set to: {default_project['name']} ({default_project['id']})", style="green")
                else:
                    console.print("Invalid index. Please enter a valid index.", style="bold red")
            except ValueError:
                console.print("Invalid input. Please enter a valid integer index.", style="bold red")
        else:
            console.print("[bold red]Error accessing API: Unable to retrieve project list.[/bold red]")

